/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)LGPL2+
 *
 * Nde - a lightweight, Qt based, desktop toolset
 * https://lxqt.org/
 *
 * Copyright: 2010-2015 Nde team
 * Authors:
 *   Paulo Lieuthier <paulolieuthier@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */

#include "leavedialog.h"
#include <QApplication>
#include <QScreen>
#include <QDebug>
#include <QThread>

LeaveDialog::LeaveDialog(QWidget* parent) : QDialog(parent)
{
    screens = qGuiApp->primaryScreen()->virtualSiblings();

    setWindowFlags((Qt::CustomizeWindowHint | Qt::FramelessWindowHint |
                    Qt::WindowStaysOnTopHint | Qt::X11BypassWindowManagerHint));
    setWindowState(Qt::WindowActive);

    for (int i = 0; i < screens.count(); i++){
        LeaveWidget *LeaveWid = new LeaveWidget(this);

        LeaveWid->setGeometry(screens.at(i)->geometry());
        LeaveWid->setFontSize();
        LeaveWid->setBackGround();
        LeaveWid->setUserAvater();
        LeaveWid->setAppsNotClose();

        connect(screens.at(i), &QScreen::geometryChanged, LeaveWid, [LeaveWid](const QRect &r){
            LeaveWid->setGeometry(r);
        });

        connect(LeaveWid, SIGNAL(accepted()),this, SLOT(accept()));
        connect(LeaveWid, SIGNAL(rejected()),this, SLOT(reject()));
        leaveWidList.append(LeaveWid);
    }

    for (int i = 0; i < screens.count(); i++)
    {
        for (int j = 0; j < screens.count(); j++)
        {
            if (i == j)
                continue;

            connect(leaveWidList.at(i), SIGNAL(buttonClick()),leaveWidList.at(j), SLOT(showUnCloseedApp()));
        }
    }

    connect(qGuiApp, &QGuiApplication::screenAdded, this, [this](QScreen *s){
        addScreenEvent(s);
    });

    connect(qGuiApp, &QGuiApplication::screenRemoved, this, [this](QScreen *s){
        deleteScreenEvent(s);
    });


}

LeaveDialog::~LeaveDialog()
{
    for (int i = 0; i < leaveWidList.count(); i++){
        if (leaveWidList.at(i))
            delete leaveWidList.at(i);
    }
}

void LeaveDialog::setConfirmText(LeaveWidget::ActionType type)
{
    for (int i = 0; i < leaveWidList.count(); i++){
        leaveWidList.at(i)->setConfirmText(type);
    }
    currentActionType = type;
}

void LeaveDialog::addScreenEvent(QScreen *s)
{
    LeaveWidget *LeaveWid = new LeaveWidget();

    LeaveWid->setGeometry(s->geometry());
    LeaveWid->setFontSize();
    LeaveWid->setBackGround();
    LeaveWid->setUserAvater();
    LeaveWid->setAppsNotClose();
    LeaveWid->setConfirmText(currentActionType);
    LeaveWid->show();

    connect(s, &QScreen::geometryChanged, LeaveWid, [LeaveWid](const QRect &r){
        LeaveWid->setGeometry(r);
    });

    connect(LeaveWid, SIGNAL(accepted()),this, SLOT(accept()));
    connect(LeaveWid, SIGNAL(rejected()),this, SLOT(reject()));

    leaveWidList.append(LeaveWid);
    screens.append(s);

    for (int i = 0; i < leaveWidList.count(); i++)
    {
        for (int j = 0; j < leaveWidList.count(); j++)
        {
            if (i == j)
                continue;

            connect(leaveWidList.at(i), SIGNAL(buttonClick()),leaveWidList.at(j), SLOT(showUnCloseedApp()));
        }
    }

    update();
}

void LeaveDialog::deleteScreenEvent(QScreen *s)
{
    if (screens.contains(s))
    {
        if (leaveWidList.at(screens.indexOf(s)))
            delete leaveWidList.at(screens.indexOf(s));
        leaveWidList.removeAt(screens.indexOf(s));

        screens.removeAt(screens.indexOf(s));
    }
}
