#include "leavewidget.h"
#include <QApplication>
#include <QString>
#include <QDebug>
#include <QSettings>
#include <QStandardPaths>
#include <KWindowEffects>
#include <QPainter>
#include <KWindowSystem/KWindowSystem>
#include <QGraphicsPixmapItem>
#include <QGraphicsBlurEffect>
#include <QDBusMessage>
#include <QDBusConnection>
#include <QKeyEvent>

#define BUTTON_MIN_HEIGHT                   32
#define LABEL_HEAD_AVATAR_MIN_HEIGHT        113
#define APP_DELETE_FREQUENCY_MS             1000
#define BACKGROUND_OFFSET_MAG               10
#define BACKGROUND_BLUR_EFFECT              40

LeaveWidget::LeaveWidget(QWidget* parent)
    : QDialog(parent)
{
    setWindowFlags((Qt::CustomizeWindowHint | Qt::FramelessWindowHint |
                    Qt::WindowStaysOnTopHint | Qt::X11BypassWindowManagerHint));

    label_bg = new QLabel(this);

    QGraphicsBlurEffect *m_effect = new QGraphicsBlurEffect(this);
    m_effect->setBlurRadius(BACKGROUND_BLUR_EFFECT);
    label_bg->setGraphicsEffect(m_effect);
    label_bg->setStyleSheet("QLabel {background-color: transparent;}");

    mpShadeWindow = new QWidget(this);
    QString str("QWidget{background-color:rgba(0,0,0,0.2);}");
    mpShadeWindow->setStyleSheet(str);
    mpShadeWindow->show();

    label_bg->show();
    label_head = new QLabel(this);

    label_username = new QLabel(qgetenv("USER"),this);
    label_username->setAlignment(Qt::AlignCenter);
    label_username->setStyleSheet("QLabel {background-color: transparent; color:#ffffff;}");

    label_p1 = new QLabel(this);
    label_p1->setAlignment(Qt::AlignLeft);
    label_p1->setStyleSheet("QLabel {background-color: transparent; color:#ffffff;}");
    label_p2 = new QLabel(this);
    label_p2->setAlignment(Qt::AlignLeft);
    label_p2->setStyleSheet("QLabel {background-color: transparent; color:#ffffff;}");
    label_p1->hide();
    label_p2->hide();

    list_app = new QListWidget(this);
    list_app->setStyleSheet("QListWidget {background-color: transparent;border:none;}"
                            "QListWidget::Item{color:#ffffff;}");

    list_app->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    list_app->hide();

    button_confirm = new QPushButton(tr("Confirm"),this);
    button_cancel = new QPushButton(tr("Cancel"),this);

    button_confirm->setStyleSheet("QPushButton{color:rgba(255,255,255,255);border: 2px solid rgba(1,18,34,2); border-radius: 5px;background-color:rgba(44,157,252,200);}"
                                  "QPushButton:hover{border: 1px solid rgba(44,157,252,255);background-color:rgba(44,157,252,255)}"
                                  "QPushButton:pressed{padding-left:1px;padding-top:2px;color:#ffffff; border:  1px solid rgba(44,157,252,255); background-color:rgba(44,157,252,255);}");
    button_cancel->setStyleSheet("QPushButton{color:rgba(255,255,255,255);border: 2px solid rgba(1,18,34,2); border-radius: 5px;background-color:rgba(44,157,252,200);}"
                                 "QPushButton:hover{border: 1px solid rgba(44,157,252,255);background-color:rgba(44,157,252,255)}"
                                 "QPushButton:pressed{padding-left:1px;padding-top:2px;color:#ffffff; border:  1px solid rgba(44,157,252,255); background-color:rgba(44,157,252,255);}");

    button_yes = new QPushButton(tr("Close"),this);
    button_no = new QPushButton(tr("Cancel"),this);

    button_yes->setStyleSheet("QPushButton{color:rgba(255,255,255,255);border: 2px solid rgba(1,18,34,2); border-radius: 5px;background-color:rgba(44,157,252,200);}"
                              "QPushButton:hover{border: 1px solid rgba(44,157,252,255);background-color:rgba(44,157,252,255)}"
                              "QPushButton:pressed{padding-left:1px;padding-top:2px;color:#ffffff; border:  1px solid rgba(44,157,252,255); background-color:rgba(44,157,252,255);}");
    button_no->setStyleSheet("QPushButton{color:rgba(255,255,255,255);border: 2px solid rgba(1,18,34,2); border-radius: 5px;background-color:rgba(44,157,252,200);}"
                             "QPushButton:hover{border: 1px solid rgba(44,157,252,255);background-color:rgba(44,157,252,255)}"
                             "QPushButton:pressed{padding-left:1px;padding-top:2px;color:#ffffff; border:  1px solid rgba(44,157,252,255); background-color:rgba(44,157,252,255);}");

    button_yes->hide();
    button_no->hide();

    for (int i = 0; i < 2; i++)
    {
        label_line[i] = new QLabel(this);
        label_line[i]->setStyleSheet("QLabel {background-color: rgba(255,255,255,25);}");
        label_line[i]->hide();
    }

    connect(button_confirm, SIGNAL(clicked()), this, SLOT(buttonClickEvent()));
    connect(button_cancel, SIGNAL(clicked()), this, SLOT(reject()));

    connect(button_yes, SIGNAL(clicked()), this, SLOT(accept()));
    connect(button_no, SIGNAL(clicked()), this, SLOT(reject()));

    connect(KWindowSystem::self(), SIGNAL(windowRemoved(WId)),this, SLOT(updateWindowApp(WId)));

    timer_delete = new QTimer(this);
    connect(timer_delete, SIGNAL(timeout()), this, SLOT(deleteItem()));
    timer_delete->stop();

    button_confirm->installEventFilter(this);
    button_cancel->installEventFilter(this);

    button_no->installEventFilter(this);
    button_yes->installEventFilter(this);

}

LeaveWidget::~LeaveWidget()
{

}

void LeaveWidget::setFontSize()
{
    if (this->height() <= 600){
        ft.setPointSize(8);
    }else if (this->height() <= 1024){
        ft.setPointSize(18);
    }else if (this->height() > 1024 && this->height() <= 1400){
        ft.setPointSize(20);
    }else if (this->height() > 1400 && this->height() <= 1600){
        ft.setPointSize(22);
    }else {
        ft.setPointSize(24);
    }
    label_username->setFont(ft);


    if (this->height() <= 600){
        ft.setPointSize(10);
    }else if (this->height() < 1024){
        ft.setPointSize(16);
    }else if (this->height() >= 1024 && this->height() < 1400){
        ft.setPointSize(20);
    }else if (this->height() >= 1400 && this->height() < 1600){
        ft.setPointSize(22);
    }else {
        ft.setPointSize(24);
    }

    label_p1->setFont(ft);


    ft.setPointSize(ft.pointSize()-8);
    label_p2->setFont(ft);

    if (ft.pointSize() < 4)
        ft.setPointSize(4);

    button_no->setFont(ft);
    button_yes->setFont(ft);
    button_confirm->setFont(ft);
    button_cancel->setFont(ft);

}

void LeaveWidget::setBackGround()
{
    //get desktop wallpaper
    QString filePath = tr("/home/") + qgetenv("USER") + tr("/.config/nde-fileman/nde/settings.conf");
    QSettings settings(filePath, QSettings::IniFormat);
    settings.beginGroup(tr("Desktop"));
    QString wallpaper = settings.value(tr("Wallpaper")).toString();
    QString argSheet = tr("background-image:url(") + wallpaper + tr(");");
    this->setAutoFillBackground(true);
    QPixmap pix(wallpaper);
    if (pix.isNull())
    {
        pix.load("/usr/share/backgrounds/default.png");
    }

    pix = pix.scaled(this->width()+this->width()/BACKGROUND_OFFSET_MAG,this->height()+this->height()/BACKGROUND_OFFSET_MAG, Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
    label_bg->setPixmap(pix);
}

void LeaveWidget::setAppsNotClose()
{
    QByteArray wCN;
    QString name;

    if (ft.pointSize() < 0)
        ft.setPointSize(10);

    for (auto wid: KWindowSystem::self()->windows())
    {
        KWindowInfo info(wid,0,NET::WM2WindowClass);
        if (info.valid())
        {
            wCN = info.windowClassName();
            name = wCN;
            if (name.compare("desktopWidget") == 0 || name.compare("nde-fileman") == 0 || name.compare("nde-panel") == 0 ) {
                continue;
            }

            currentWIds.append(wid);
            appIcons.append(KWindowSystem::self()->icon(wid));
            appName.append(name);

            QListWidgetItem *app = new QListWidgetItem(list_app);
            app->setIcon(KWindowSystem::self()->icon(wid));
            app->setText("       "+name);
            app->setSizeHint(QSize(this->width()*3/5, this->height()*3/50));
            app->setFlags(app->flags() &~Qt::ItemIsSelectable &~Qt::ItemIsEditable &~Qt::ItemIsDragEnabled &~Qt::ItemIsDropEnabled &~Qt::ItemIsUserCheckable);


            app->setFont(ft);
            list_app->addItem(app);
        }
    }
}

void LeaveWidget::setUserAvater()
{
    //set user avatar
    QPixmap pixmapa("/usr/share/sddm/faces/.face.icon");
    QPixmap pixmap(label_head->height(),label_head->height());
    pixmap.fill(Qt::transparent);
    QPainter painter(&pixmap);
    painter.setRenderHints(QPainter::Antialiasing | QPainter::SmoothPixmapTransform);
    QPainterPath path;
    path.addEllipse(0, 0, label_head->height(), label_head->height());
    painter.setClipPath(path);
    painter.drawPixmap(0, 0, label_head->height(), label_head->height(), pixmapa);
    label_head->setPixmap(pixmap);
}

void LeaveWidget::setGeometry(QRect rect)
{
    QWidget::setGeometry(rect);
    mpShadeWindow->setGeometry(rect);
    list_app->setIconSize(QSize(this->height()*3/70, this->height()*3/70));

    int bg_offset_wid = this->width()/BACKGROUND_OFFSET_MAG;
    int bg_offset_hei = this->height()/BACKGROUND_OFFSET_MAG;
    label_bg->setGeometry(bg_offset_wid*(-1)/2,bg_offset_hei*(-1)/2,this->width()+bg_offset_wid,this->height()+bg_offset_hei);
    int buttonHeight = this->height() / 32;
//    if (buttonHeight < BUTTON_MIN_HEIGHT){
//        buttonHeight = BUTTON_MIN_HEIGHT;
//    }
    int buttonWidth = buttonHeight * 100 / 32 ;
    int button_interval = 8;


    int labelHeadLen = this->height() / 9;
//    if (labelHeadLen < LABEL_HEAD_AVATAR_MIN_HEIGHT)
//        labelHeadLen = LABEL_HEAD_AVATAR_MIN_HEIGHT;

    int labelUserNameWid = (buttonWidth+button_interval)*10;
    int labelUserNameHei = buttonHeight;

    int startY = this->height()*2/5;

    label_head->setGeometry(this->width()/2-labelHeadLen/2,startY - buttonHeight,labelHeadLen,labelHeadLen);
    label_username->setGeometry(this->width()/2-(labelUserNameWid/2),startY+labelHeadLen-buttonHeight-button_interval,labelUserNameWid,labelUserNameHei*2);
    button_confirm->setGeometry(this->width()/2-button_interval-buttonWidth,startY+labelHeadLen+2*buttonHeight,buttonWidth,buttonHeight);
    button_cancel->setGeometry(this->width()/2+button_interval,startY+labelHeadLen+2*buttonHeight,buttonWidth,buttonHeight);

    //---------------
    int startY1 = this->height()/4;
    int startX = this->width()/5;

    int labelP1Hei = this->height()/20;
    int listAppHei = this->height()*3/10;
    int listAppWid = this->width()*3/5;

    label_p1->setGeometry(startX, startY1-button_interval, listAppWid, labelP1Hei);
    label_p2->setGeometry(startX, startY1 + labelP1Hei/2, listAppWid, labelP1Hei);
    list_app->setGeometry(startX, startY1 + labelP1Hei*2, listAppWid, listAppHei);

    button_yes->setGeometry(this->width()*4/5-buttonWidth*2-button_interval, startY1+labelP1Hei*6/2+listAppHei,buttonWidth,buttonHeight);
    button_no->setGeometry(this->width()*4/5-buttonWidth, startY1+labelP1Hei*6/2+listAppHei,buttonWidth,buttonHeight);

    label_line[0]->setGeometry(startX, startY1+labelP1Hei*3/2, listAppWid, 1);
    label_line[1]->setGeometry(startX, startY1+labelP1Hei*5/2+listAppHei, listAppWid, 1);
}


void LeaveWidget::setGeometry(int ax, int ay, int aw, int ah)
{

    QWidget::setGeometry(ax,ay,aw,ah);


    int bg_offset_wid = this->width()/BACKGROUND_OFFSET_MAG;
    int bg_offset_hei = this->height()/BACKGROUND_OFFSET_MAG;
    label_bg->setGeometry(bg_offset_wid*(-1)/2,bg_offset_hei*(-1)/2,this->width()+bg_offset_wid,this->height()+bg_offset_hei);
    int buttonHeight = this->height() / 32;
    if (buttonHeight < BUTTON_MIN_HEIGHT){
        buttonHeight = BUTTON_MIN_HEIGHT;
    }
    int buttonWidth = buttonHeight * 100 / 32 ;
    int button_interval = 8;


    int labelHeadLen = this->height() / 9;
    if (labelHeadLen < LABEL_HEAD_AVATAR_MIN_HEIGHT)
        labelHeadLen = LABEL_HEAD_AVATAR_MIN_HEIGHT;

    int labelUserNameWid = (buttonWidth+button_interval)*10;
    int labelUserNameHei = buttonHeight;

    int startY = this->height()*2/5;

    label_head->setGeometry(this->width()/2-labelHeadLen/2,startY - buttonHeight,labelHeadLen,labelHeadLen);
    label_username->setGeometry(this->width()/2-(labelUserNameWid/2),startY+labelHeadLen-buttonHeight-button_interval,labelUserNameWid,labelUserNameHei*2);
    mpShadeWindow->setGeometry(0,0,this->width(),this->height());
    button_confirm->setGeometry(this->width()/2-button_interval-buttonWidth,startY+labelHeadLen+2*buttonHeight,buttonWidth,buttonHeight);
    button_cancel->setGeometry(this->width()/2+button_interval,startY+labelHeadLen+2*buttonHeight,buttonWidth,buttonHeight);

    //---------------
    int startY1 = this->height()/4;
    int startX = this->width()/5;

    int labelP1Hei = this->height()/20;
    int listAppHei = this->height()*3/10;
    int listAppWid = this->width()*3/5;

    label_p1->setGeometry(startX, startY1-button_interval, listAppWid, labelP1Hei);
    label_p2->setGeometry(startX, startY1 + labelP1Hei/2, listAppWid, labelP1Hei);
    list_app->setGeometry(startX, startY1 + labelP1Hei*2, listAppWid, listAppHei);

    button_yes->setGeometry(this->width()*4/5-buttonWidth*2-button_interval, startY1+labelP1Hei*6/2+listAppHei,buttonWidth,buttonHeight);
    button_no->setGeometry(this->width()*4/5-buttonWidth, startY1+labelP1Hei*6/2+listAppHei,buttonWidth,buttonHeight);

    label_line[0]->setGeometry(startX, startY1+labelP1Hei*3/2, listAppWid, 1);
    label_line[1]->setGeometry(startX, startY1+labelP1Hei*5/2+listAppHei, listAppWid, 1);
}

void LeaveWidget::updateWindowApp(WId id)
{
    QByteArray wCN;
    QString name;
    list_app->clear();
    int count = 0;

    for (auto wid: KWindowSystem::self()->windows())
    {
        KWindowInfo info(wid,0,NET::WM2WindowClass);
        if (info.valid())
        {
            wCN = info.windowClassName();
            name = wCN;
            if (name.compare("desktopWidget") == 0 || name.compare("nde-fileman") == 0 || name.compare("nde-panel") == 0 ) {
                continue;
            }
            count++;
            QListWidgetItem *app = new QListWidgetItem(list_app);
            app->setIcon(KWindowSystem::self()->icon(wid));
            app->setText("       "+name);
            app->setSizeHint(QSize(this->width()*3/5, this->height()*3/50));
            app->setFlags(app->flags() & ~Qt::ItemIsSelectable);

            app->setFont(ft);
            list_app->addItem(app);
        }
    }

    QString p1 = tr("Now close ") + QString("%1").arg(count) + tr(" applications and ") + button_confirm->text();
    label_p1->setText(p1);

    if (count == 0){
        QDialog::accept();
    }
}

void LeaveWidget::deleteItem()
{
    timer_delete->stop();
    static int index = 0;
    if (index >= currentWIds.count())
        return ;

    WId id = currentWIds.at(index);
    index++;

    QDBusMessage message = QDBusMessage::createMethodCall("com.kde.kwin",
                           "/nde",
                           "com.kde.KWin",
                           "removeWindow");
    QList<QVariant> args;
    args.insert(0,QString::number(id));
    message.setArguments(args);
    //发送消息
    QDBusMessage response = QDBusConnection::sessionBus().call(message);

    timer_delete->start(APP_DELETE_FREQUENCY_MS);
}

void LeaveWidget::buttonClickEvent()
{
    QPushButton *btn = qobject_cast<QPushButton*>(sender());

    if (btn == button_confirm){
        if (appName.count() != 0 && button_confirm->text() != tr("Hibernate")){
            if (btn == button_confirm){
                emit buttonClick();
                label_head->hide();
                label_username->hide();
                button_confirm->hide();
                button_cancel->hide();

                label_p1->show();
                label_p2->show();
                list_app->show();
                button_yes->show();
                button_no->show();
                label_line[0]->show();
                label_line[1]->show();

                update();
                timer_delete->start(APP_DELETE_FREQUENCY_MS);
            }else{
                QDialog::reject();
            }
        }else {
            QDialog::accept();
        }
    }
}

void LeaveWidget::showUnCloseedApp()
{
    label_head->hide();
    label_username->hide();
    button_confirm->hide();
    button_cancel->hide();

    label_p1->show();
    label_p2->show();
    list_app->show();
    button_yes->show();
    button_no->show();
    label_line[0]->show();
    label_line[1]->show();

    update();
}

void LeaveWidget::setConfirmText(ActionType type)
{
    switch(type)
    {
        case Logout:
            button_confirm->setText(tr("Logout"));
            break;
        case Hibernate:
            button_confirm->setText(tr("Hibernate"));
            break;
        case Reboot:
            button_confirm->setText(tr("Reboot"));
            break;
        case Shutdown:
            button_confirm->setText(tr("Shutdown"));
            break;
        case Switchuser:
            button_confirm->setText(tr("Switchuser"));
            break;
    }

    QString p1 = tr("Now close ") + QString("%1").arg(appName.count()) + tr(" applications and ") + button_confirm->text();
    QString p2 = tr("If you want to go back and save your work, click 'Cancel' and then complete the operation you want to perform.");

    button_yes->setText(tr("Force ") + button_confirm->text());
    label_p1->setText(p1);
    label_p2->setText(p2);
}

void LeaveWidget::showEvent(QShowEvent *)
{
    label_head->show();
    label_username->show();
    button_confirm->show();
    button_cancel->show();
}

bool LeaveWidget::eventFilter(QObject *target, QEvent *event)
{
    if(event->type() == QEvent::KeyPress)
    {
        QKeyEvent *keyEvent = static_cast<QKeyEvent*>(event);
        if (keyEvent->key() == Qt::Key_Return)
        {
            if (!button_yes->isHidden())
                button_yes->click();
            else if (!button_confirm->isHidden())
                button_confirm->click();

            return true;
        }
        else if (keyEvent->key() == Qt::Key_Escape)
        {
            if (!button_no->isHidden())
                button_no->click();
            else if (!button_cancel->isHidden())
                button_cancel->click();

            return true;
        }
    }

    return QDialog::eventFilter(target, event);
}
